const { Vector, Rotator, ProgressBar, Button, UIElement, refObject } = require('@tabletop-playground/api');

function addHitPointCounter(hitPointObj, maxHitPoints) 
{
	var defHP = 12;
    function setBarValue(obj) 
	{
        obj.progressBar
            .setText((`${obj.hitPoints}/${obj.maxHitPoints}`))
            .setProgress(obj.hitPoints / obj.maxHitPoints);
    }
    
    function minusPressed(button, player) 
	{
        var owner = button.getOwningObject();
        if (owner.hitPoints <= 1) 
		{
            // Final hit point lost, die!
            //owner.destroy();
			// show a message on screen 
			owner.hitPoints = 0;
			player.showMessage("Crossbones got Smacked!");
			setBarValue(owner);
		}
        else 
		{
            owner.hitPoints--;
            setBarValue(owner);
            // Store hitpoints in game states (number is automatically converted to string) 
            owner.setSavedData(owner.hitPoints);
        }
    }
    
    function plusPressed(button, player) 
	{
        var owner = button.getOwningObject();
        owner.hitPoints = Math.min(owner.hitPoints + 1, owner.maxHitPoints);
        setBarValue(owner);
    }
	
	function incMaxPressed(button, player) 
	{
	var owner = button.getOwningObject();
        owner.maxHitPoints = Math.min(owner.maxHitPoints + 1);
		owner.hitPoints = Math.min(owner.hitPoints + 1, owner.maxHitPoints);
        setBarValue(owner);
    }
	
	function decMaxPressed(button, player) 
	{
	var owner = button.getOwningObject();
	
	if (owner.hitPoints <= 1) 
		{
            // Final hit point lost, die!
            //owner.destroy();
			// show a message on screen 
			player.showMessage("Crossbones got Smacked!");
		}
		else
		{
		
			if (owner.maxHitPoints <= defHP)
			{
				owner.maxHitPoints = defHP;
			}
			else
			{
				owner.maxHitPoints = Math.min(owner.maxHitPoints - 1);
				owner.hitPoints = Math.min(owner.hitPoints - 1, owner.maxHitPoints);
				setBarValue(owner);				
			}
		}   
    }
//	else
//		{
//		
//			if (owner.maxHitPoints <= defHitPoints)
//			{
//				owner.maxHitPoints = defHitPoints;
//			}
//			else
//			{
//				if (owner.hitPoints > 11)
//				{
//				
//				owner.maxHitPoints = Math.min(owner.maxHitPoints - 1);
//				owner.hitPoints = Math.min(owner.hitPoints - 1, owner.maxHitPoints);
//				setBarValue(owner);
//				}
//				else
//				{
//					owner.maxHitPoints = Math.min(owner.maxHitPoints - 1);
//					setBarValue(owner);
//				}
//			}
//		}   
 //   }
	
    // Add hit point variables to the object
    hitPointObj.maxHitPoints = maxHitPoints;
    // If this object is loaded from a game state or copied from another object, it may 
    // already have a stored hit point value.
    hitPointObj.hitPoints = parseInt(refObject.getSavedData());
    if (isNaN(hitPointObj.hitPoints)) 
	{
        hitPointObj.hitPoints = maxHitPoints;
    }
	
    // Create widgets
    hitPointObj.progressBar = new ProgressBar();
    setBarValue(hitPointObj);
    
    var minusButton = new Button().setText("-");
    minusButton.onClicked.add(minusPressed);
    
    var plusButton = new Button().setText("+");
    plusButton.onClicked.add(plusPressed);
	
	var incMaxButton = new Button().setText(">");
    incMaxButton.onClicked.add(incMaxPressed);
	
	var decMaxButton = new Button().setText("<");
    decMaxButton.onClicked.add(decMaxPressed);
    
    // Add the three UI elements next to each other at an angle
    var ui = new UIElement();
    ui.useWidgetSize = false;
    ui.width = 60;
    ui.height = 30;
    ui.position = new Vector(-3.6, 0, 0.2);
    ui.rotation = new Rotator(25, 0, 0);
    ui.scale = 0.5;
    ui.widget = refObject.progressBar;
    hitPointObj.attachUI(ui);
    
    var ui2 = new UIElement();
    ui2.useWidgetSize = false;
    ui2.width = 25;
    ui2.height = 35;
    ui2.position = new Vector(-3.6, -1.85, 0.2);
    ui2.rotation = new Rotator(25, 0, 0);
    ui2.scale = 0.4;
    ui2.widget = minusButton;
    hitPointObj.attachUI(ui2);
    
    var ui3 = new UIElement();
    ui2.useWidgetSize = false;
    ui2.width = 25;
    ui2.height = 35;
    ui3.position = new Vector(-3.6, 1.85, 0.2);
    ui3.rotation = new Rotator(25, 0, 0);
    ui3.scale = 0.4;
    ui3.widget = plusButton;
    hitPointObj.attachUI(ui3);
	
	var ui4 = new UIElement();
    ui2.useWidgetSize = false;
    ui2.width = 25;
    ui2.height = 35;
    ui4.position = new Vector(-3.6, 2.7, 0.2);
    ui4.rotation = new Rotator(25, 0, 0);
    ui4.scale = 0.4;
    ui4.widget = incMaxButton;
    hitPointObj.attachUI(ui4);
	
	var ui5 = new UIElement();
    ui2.useWidgetSize = false;
    ui2.width = 25;
    ui2.height = 35;
    ui5.position = new Vector(-3.6, -2.7, 0.2);
    ui5.rotation = new Rotator(25, 0, 0);
    ui5.scale = 0.4;
    ui5.widget = decMaxButton;
    hitPointObj.attachUI(ui5);
	
	
}
// Add a counter for 10 hit points to our reference object
addHitPointCounter(refObject, 12);